Type: Package
Package: accessibility
Title: Transport Accessibility Measures
Version: 1.4.0
Authors@R: c(
    person("Rafael H. M.", "Pereira", , "rafa.pereira.br@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-2125-7465")),
    person("Daniel", "Herszenhut", , "dhersz@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8066-1105")),
    person("Joey", "Reid", role = "ctb"),
    person("Ipea - Institute for Applied Economic Research", role = c("cph", "fnd"))
  )
Description: A set of fast and convenient functions to help conducting
    accessibility analyses. Given a pre-computed travel cost matrix and a
    land use dataset (containing the location of jobs, healthcare and
    population, for example), the package allows one to calculate
    accessibility levels and accessibility poverty and inequality. The
    package covers the majority of the most commonly used accessibility
    measures (such as cumulative opportunities, gravity-based and floating
    catchment areas methods), as well as the most frequently used
    inequality and poverty metrics (such as the Palma ratio, the
    concentration and Theil indices and the FGT family of measures).
License: MIT + file LICENSE
URL: https://github.com/ipeaGIT/accessibility,
        https://ipeagit.github.io/accessibility/
BugReports: https://github.com/ipeaGIT/accessibility/issues
Depends: R (>= 3.5.0)
Imports: checkmate, data.table, Hmisc, Rdpack (>= 0.7), stats, utils
Suggests: covr, ggplot2, knitr, rmarkdown, sf, testthat, tibble
VignetteBuilder: knitr
RdMacros: Rdpack
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.2.3
Packaged: 2024-03-06 20:44:17 UTC; dhersz
Author: Rafael H. M. Pereira [aut] (<https://orcid.org/0000-0003-2125-7465>),
  Daniel Herszenhut [aut, cre] (<https://orcid.org/0000-0001-8066-1105>),
  Joey Reid [ctb],
  Ipea - Institute for Applied Economic Research [cph, fnd]
Maintainer: Daniel Herszenhut <dhersz@gmail.com>
Repository: CRAN
Date/Publication: 2024-03-06 23:40:06 UTC
Built: R 4.2.3; ; 2024-03-07 01:47:31 UTC; unix
