Type: Package
Package: accessibility
Title: Transport Accessibility Measures
Version: 1.5.0
Authors@R: c(
    person("Rafael H. M.", "Pereira", , "rafa.pereira.br@gmail.com", role = "aut",
           comment = c(ORCID = "0000-0003-2125-7465")),
    person("Daniel", "Herszenhut", , "dhersz@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-8066-1105")),
    person("Anastasia", "Soukhov", role = "aut"),
    person("Christopher", "Higgins", role = "ctb"),
    person("Joey", "Reid", role = "ctb"),
    person("Ipea - Institute for Applied Economic Research", role = c("cph", "fnd"))
  )
Description: A set of fast and convenient functions to help conducting 
    accessibility analyses. Given a pre-computed travel cost matrix and a land 
    use dataset (containing for example the location of jobs, healthcare and 
    population), the package allows one to calculate accessibility levels, and 
    accessibility poverty and inequality. The package covers the majority of the 
    most commonly used accessibility measures (such as cumulative opportunities, 
    gravity-based and floating catchment areas methods), some cutting edge 
    measures proposed in the literature (e.g. balancing cost and constrained 
    accessibility) as well as the most frequently used inequality and poverty 
    metrics (such as the Palma ratio, the concentration and Theil indices and 
    the FGT family of measures).
License: MIT + file LICENSE
URL: https://github.com/ipeaGIT/accessibility,
        https://ipeagit.github.io/accessibility/
BugReports: https://github.com/ipeaGIT/accessibility/issues
Depends: R (>= 3.5.0)
Imports: checkmate, data.table, Hmisc, Rdpack (>= 0.7), stats, utils
Suggests: covr, ggplot2, knitr, rmarkdown, sf, testthat, tibble
VignetteBuilder: knitr
RdMacros: Rdpack
Encoding: UTF-8
NeedsCompilation: no
RoxygenNote: 7.3.3
Packaged: 2026-05-12 00:49:29 UTC; rafap
Author: Rafael H. M. Pereira [aut] (ORCID:
    <https://orcid.org/0000-0003-2125-7465>),
  Daniel Herszenhut [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8066-1105>),
  Anastasia Soukhov [aut],
  Christopher Higgins [ctb],
  Joey Reid [ctb],
  Ipea - Institute for Applied Economic Research [cph, fnd]
Maintainer: Daniel Herszenhut <dhersz@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-12 05:10:21 UTC
Built: R 4.6.0; ; 2026-05-12 07:08:21 UTC; unix
