# Generated by roxygen2: do not edit by hand

export(balancing_cost)
export(concentration_index)
export(constrained_accessibility)
export(cost_to_closest)
export(cumulative_cutoff)
export(cumulative_interval)
export(decay_binary)
export(decay_exponential)
export(decay_linear)
export(decay_logistic)
export(decay_power)
export(decay_stepped)
export(fgt_poverty)
export(floating_catchment_area)
export(gini_index)
export(gravity)
export(palma_ratio)
export(spatial_availability)
export(theil_t)
importFrom(Rdpack,reprompt)
importFrom(data.table,"%chin%")
importFrom(data.table,":=")
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(utils,globalVariables)
