#' @param travel_matrix A data frame. The travel matrix describing the costs
#'   (i.e. travel time, distance, monetary cost, etc.) between the origins and
#'   destinations in the study area. Must contain the columns `from_id`, `to_id`
#'   and any others specified in `travel_cost`.
