#' @param active A logical. When `TRUE`, the function calculates active
#'        accessibility (the quantity of opportunities that can be reached from
#'        a given origin). when `FALSE`, it calculates passive accessibility (by
#'        how many people each destination can be reached), which is equivalent
#'        to the notion of market potential.
