#' @param travel_cost A string. The name of the column in `travel_matrix` with
#'        the travel cost between origins and destinations. The notion of cost
#'        here is generic over any kind of numeric travel cost, such as distance,
#'        time and money.
