#' @param data A `data.frame` with a travel cost matrix in long format,
#'   containing the columns with origin ids, destination ids, the travel cost
#'   between each origin-destination pair (this can be either in  terms of
#'   travel time, distance or monetary costs), and number of population and
#'   opportunities located in each origin and destination.
