% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/singly_constrained.R
\name{singly_constrained}
\alias{singly_constrained}
\title{Singly constrained accessibility}
\value{
A \code{data.table}/\code{data.frame} with results (structure mirrors the wrapper).
}
\description{
Allocates opportunities at each destination proportionally based on travel
impedance and population at the origin. Uses the logic of Wilson's single
constraint. Returns values in the unit of 'demand'.
Internal helper function used by \code{\link[=constrained_accessibility]{constrained_accessibility()}} when \code{constraint = "singly"}.
}
\examples{
NULL
}
\keyword{internal}
