% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/total_constrained.R
\name{total_constrained}
\alias{total_constrained}
\title{Total constrained accessibility}
\value{
A \code{data.table}/\code{data.frame} with results (structure mirrors the wrapper).
}
\description{
Allocates total opportunities in the region proportionally based on travel
impedance. Uses the logic of a total (or unconstrained by Wilson's terms)
constraint. Returns values in units of 'supply' (i.e., opportunities) if
\code{active = TRUE} and returns values in units of demand' (i.e., population)
if \code{active = FALSE}.
}
\details{
Internal helper used by \code{\link[=constrained_accessibility]{constrained_accessibility()}} when \code{constraint = "total"}.
}
\examples{
NULL

}
\keyword{internal}
